@@ -13,6 +13,7 @@ from dagster import check, seven
 from dagster.cli.workspace.cli_target import (
     get_repository_location_from_kwargs,
     get_repository_origin_from_kwargs,
+    get_working_directory_from_kwargs,
     python_origin_target_argument,
     repository_target_argument,
 )
@@ -58,10 +59,10 @@ from dagster.core.storage.tags import check_tags
 from dagster.core.types.loadable_target_origin import LoadableTargetOrigin
 from dagster.grpc import DagsterGrpcServer
 from dagster.grpc.impl import (
+    get_external_executable_params,
     get_external_execution_plan_snapshot,
     get_external_pipeline_subset_result,
     get_external_schedule_execution,
-    get_external_triggered_execution_params,
     get_partition_config,
     get_partition_names,
     get_partition_set_execution_param_data,
@@ -71,8 +72,8 @@ from dagster.grpc.types import (
     ExecuteRunArgs,
     ExecuteStepArgs,
     ExecutionPlanSnapshotArgs,
+    ExternalExecutableArgs,
     ExternalScheduleExecutionArgs,
-    ExternalTriggeredExecutionArgs,
     ListRepositoriesInput,
     ListRepositoriesResponse,
     LoadableRepositorySymbol,
@@ -277,17 +278,17 @@ def schedule_execution_data_command(args):
 
 
 @unary_api_cli_command(
-    name="trigger_execution_params",
+    name="executable_params",
     help_str=(
         "[INTERNAL] Return the execution params for a triggered execution. This is an internal "
         "utility. Users should generally not invoke this command interactively."
     ),
-    input_cls=ExternalTriggeredExecutionArgs,
+    input_cls=ExternalExecutableArgs,
     output_cls=(ExternalExecutionParamsData, ExternalExecutionParamsErrorData),
 )
-def trigger_execution_params_command(args):
+def executable_params_command(args):
     recon_repo = recon_repository_from_origin(args.repository_origin)
-    return get_external_triggered_execution_params(recon_repo, args)
+    return get_external_executable_params(recon_repo, args)
 
 
 @whitelist_for_serdes
@@ -307,16 +308,15 @@ class ExecuteRunArgsLoadComplete(namedtuple("_ExecuteRunArgsLoadComplete", "")):
 def execute_run_command(input_file, output_file):
     args = check.inst(read_unary_input(input_file), ExecuteRunArgs)
     recon_pipeline = recon_pipeline_from_origin(args.pipeline_origin)
-    instance = DagsterInstance.from_ref(args.instance_ref)
+    with DagsterInstance.from_ref(args.instance_ref) as instance:
+        with ipc_write_stream(output_file) as ipc_stream:
 
-    with ipc_write_stream(output_file) as ipc_stream:
+            def send_to_stream(event):
+                ipc_stream.send(event)
 
-        def send_to_stream(event):
-            ipc_stream.send(event)
-
-        return _execute_run_command_body(
-            recon_pipeline, args.pipeline_run_id, instance, send_to_stream
-        )
+            return _execute_run_command_body(
+                recon_pipeline, args.pipeline_run_id, instance, send_to_stream
+            )
 
 
 @click.command(
@@ -333,19 +333,18 @@ def execute_run_with_structured_logs_command(input_json):
     args = check.inst(deserialize_json_to_dagster_namedtuple(input_json), ExecuteRunArgs)
     recon_pipeline = recon_pipeline_from_origin(args.pipeline_origin)
 
-    instance = (
+    with (
         DagsterInstance.from_ref(args.instance_ref) if args.instance_ref else DagsterInstance.get()
-    )
+    ) as instance:
+        buffer = []
 
-    buffer = []
+        def send_to_buffer(event):
+            buffer.append(serialize_dagster_namedtuple(event))
 
-    def send_to_buffer(event):
-        buffer.append(serialize_dagster_namedtuple(event))
+        _execute_run_command_body(recon_pipeline, args.pipeline_run_id, instance, send_to_buffer)
 
-    _execute_run_command_body(recon_pipeline, args.pipeline_run_id, instance, send_to_buffer)
-
-    for line in buffer:
-        click.echo(line)
+        for line in buffer:
+            click.echo(line)
 
 
 def _execute_run_command_body(recon_pipeline, pipeline_run_id, instance, write_stream_fn):
@@ -406,29 +405,31 @@ def execute_step_with_structured_logs_command(input_json):
 
     args = check.inst(deserialize_json_to_dagster_namedtuple(input_json), ExecuteStepArgs)
 
-    instance = (
+    with (
         DagsterInstance.from_ref(args.instance_ref) if args.instance_ref else DagsterInstance.get()
-    )
-    pipeline_run = instance.get_run_by_id(args.pipeline_run_id)
-    recon_pipeline = recon_pipeline_from_origin(args.pipeline_origin)
+    ) as instance:
+        pipeline_run = instance.get_run_by_id(args.pipeline_run_id)
+        recon_pipeline = recon_pipeline_from_origin(args.pipeline_origin)
 
-    execution_plan = create_execution_plan(
-        recon_pipeline.subset_for_execution_from_existing_pipeline(pipeline_run.solids_to_execute),
-        run_config=args.run_config,
-        step_keys_to_execute=args.step_keys_to_execute,
-        mode=args.mode,
-    )
+        execution_plan = create_execution_plan(
+            recon_pipeline.subset_for_execution_from_existing_pipeline(
+                pipeline_run.solids_to_execute
+            ),
+            run_config=args.run_config,
+            step_keys_to_execute=args.step_keys_to_execute,
+            mode=args.mode,
+        )
 
-    retries = Retries.from_config(args.retries_dict)
+        retries = Retries.from_config(args.retries_dict)
 
-    buff = []
-    for event in execute_plan_iterator(
-        execution_plan, pipeline_run, instance, run_config=args.run_config, retries=retries,
-    ):
-        buff.append(serialize_dagster_namedtuple(event))
+        buff = []
+        for event in execute_plan_iterator(
+            execution_plan, pipeline_run, instance, run_config=args.run_config, retries=retries,
+        ):
+            buff.append(serialize_dagster_namedtuple(event))
 
-    for line in buff:
-        click.echo(line)
+        for line in buff:
+            click.echo(line)
 
 
 class _ScheduleTickHolder:
@@ -502,7 +503,16 @@ def _schedule_tick_state(instance, stream, tick_data):
     type=click.INT,
     required=False,
     default=30,
-    help="Timout after which to shutdown if --heartbeat is set and a heartbeat is not received",
+    help="Timeout after which to shutdown if --heartbeat is set and a heartbeat is not received",
+)
+@click.option(
+    "--lazy-load-user-code",
+    is_flag=True,
+    required=False,
+    default=False,
+    help="Wait until the first LoadRepositories call to actually load the repositoriies, instead of"
+    "waiting to load them when the server is launched. Useful for surfacing errors when the server "
+    "is managed directly from Dagit",
 )
 @python_origin_target_argument
 def grpc_command(
@@ -512,6 +522,7 @@ def grpc_command(
     max_workers=1,
     heartbeat=False,
     heartbeat_timeout=30,
+    lazy_load_user_code=False,
     **kwargs
 ):
     if seven.IS_WINDOWS and port is None:
@@ -526,7 +537,7 @@ def grpc_command(
         loadable_target_origin = LoadableTargetOrigin(
             executable_path=sys.executable,
             attribute=kwargs["attribute"],
-            working_directory=kwargs["working_directory"],
+            working_directory=get_working_directory_from_kwargs(kwargs),
             module_name=kwargs["module_name"],
             python_file=kwargs["python_file"],
         )
@@ -539,6 +550,7 @@ def grpc_command(
         max_workers=max_workers,
         heartbeat=heartbeat,
         heartbeat_timeout=heartbeat_timeout,
+        lazy_load_user_code=lazy_load_user_code,
     )
 
     server.serve()
@@ -559,47 +571,47 @@ def grpc_command(
 @click.option("--schedule_name")
 def launch_scheduled_execution(output_file, schedule_name, **kwargs):
     with ipc_write_stream(output_file) as stream:
-        instance = DagsterInstance.get()
-        repository_origin = get_repository_origin_from_kwargs(kwargs)
-        schedule_origin = repository_origin.get_schedule_origin(schedule_name)
+        with DagsterInstance.get() as instance:
+            repository_origin = get_repository_origin_from_kwargs(kwargs)
+            schedule_origin = repository_origin.get_schedule_origin(schedule_name)
 
-        # open the tick scope before we load any external artifacts so that
-        # load errors are stored in DB
-        with _schedule_tick_state(
-            instance,
-            stream,
-            ScheduleTickData(
-                schedule_origin_id=schedule_origin.get_id(),
-                schedule_name=schedule_name,
-                timestamp=time.time(),
-                cron_schedule=None,  # not yet loaded
-                status=ScheduleTickStatus.STARTED,
-            ),
-        ) as tick:
-            with get_repository_location_from_kwargs(kwargs, instance) as repo_location:
-                repo_dict = repo_location.get_repositories()
-                check.invariant(
-                    repo_dict and len(repo_dict) == 1,
-                    "Passed in arguments should reference exactly one repository, instead there are {num_repos}".format(
-                        num_repos=len(repo_dict)
-                    ),
-                )
-                external_repo = next(iter(repo_dict.values()))
-                check.invariant(
-                    schedule_name
-                    in [schedule.name for schedule in external_repo.get_external_schedules()],
-                    "Could not find schedule named {schedule_name}".format(
-                        schedule_name=schedule_name
-                    ),
-                )
-                external_schedule = external_repo.get_external_schedule(schedule_name)
-                tick.update_with_status(
+            # open the tick scope before we load any external artifacts so that
+            # load errors are stored in DB
+            with _schedule_tick_state(
+                instance,
+                stream,
+                ScheduleTickData(
+                    schedule_origin_id=schedule_origin.get_id(),
+                    schedule_name=schedule_name,
+                    timestamp=time.time(),
+                    cron_schedule=None,  # not yet loaded
                     status=ScheduleTickStatus.STARTED,
-                    cron_schedule=external_schedule.cron_schedule,
-                )
-                _launch_scheduled_execution(
-                    instance, repo_location, external_repo, external_schedule, tick, stream
-                )
+                ),
+            ) as tick:
+                with get_repository_location_from_kwargs(kwargs, instance) as repo_location:
+                    repo_dict = repo_location.get_repositories()
+                    check.invariant(
+                        repo_dict and len(repo_dict) == 1,
+                        "Passed in arguments should reference exactly one repository, instead there are {num_repos}".format(
+                            num_repos=len(repo_dict)
+                        ),
+                    )
+                    external_repo = next(iter(repo_dict.values()))
+                    check.invariant(
+                        schedule_name
+                        in [schedule.name for schedule in external_repo.get_external_schedules()],
+                        "Could not find schedule named {schedule_name}".format(
+                            schedule_name=schedule_name
+                        ),
+                    )
+                    external_schedule = external_repo.get_external_schedule(schedule_name)
+                    tick.update_with_status(
+                        status=ScheduleTickStatus.STARTED,
+                        cron_schedule=external_schedule.cron_schedule,
+                    )
+                    _launch_scheduled_execution(
+                        instance, repo_location, external_repo, external_schedule, tick, stream
+                    )
 
 
 def _launch_scheduled_execution(
@@ -622,6 +634,7 @@ def _launch_scheduled_execution(
         repository_handle=external_repo.handle,
         schedule_name=external_schedule.name,
         schedule_execution_data_mode=ScheduleExecutionDataMode.LAUNCH_SCHEDULED_EXECUTION,
+        scheduled_execution_datetime_utc=None,  # No way to know this in general for this scheduler
     )
 
     run_config = {}
@@ -726,7 +739,7 @@ def create_api_cli_group():
     group.add_command(partition_names_command)
     group.add_command(partition_set_execution_param_command)
     group.add_command(schedule_execution_data_command)
-    group.add_command(trigger_execution_params_command)
+    group.add_command(executable_params_command)
     group.add_command(launch_scheduled_execution)
     group.add_command(grpc_command)
     return group
