@@ -10,6 +10,7 @@ import yaml
 
 from dagster import PipelineDefinition, check, execute_pipeline
 from dagster.cli.workspace.cli_target import (
+    WORKSPACE_TARGET_WARNING,
     get_external_pipeline_from_external_repo,
     get_external_pipeline_from_kwargs,
     get_external_repository_from_kwargs,
@@ -20,7 +21,7 @@ from dagster.cli.workspace.cli_target import (
     python_pipeline_target_argument,
     repository_target_argument,
 )
-from dagster.core.definitions.executable import ExecutablePipeline
+from dagster.core.definitions.pipeline_base import IPipeline
 from dagster.core.errors import (
     DagsterBackfillFailedError,
     DagsterInvariantViolationError,
@@ -63,10 +64,6 @@ def create_pipeline_cli_group():
     return group
 
 
-WORKSPACE_TARGET_WARNING = "Can only use ONE of --workspace/-w, --python-file/-f, --module-name/-m."
-WORKSPACE_ARG_NAMES = ["workspace", "module_name", "python_file", "attribute"]
-
-
 def apply_click_params(command, *click_params):
     for click_param in click_params:
         command = click_param(command)
@@ -473,7 +470,7 @@ def _create_external_pipeline_run(
 def do_execute_command(
     pipeline, instance, config, mode=None, tags=None, solid_selection=None, preset=None,
 ):
-    check.inst_param(pipeline, "pipeline", ExecutablePipeline)
+    check.inst_param(pipeline, "pipeline", IPipeline)
     check.inst_param(instance, "instance", DagsterInstance)
     check.opt_list_param(config, "config", of_type=str)
 
