@@ -176,7 +176,19 @@ class DagsterEvent(
 ):
     """Events yielded by solid and pipeline execution.
 
-    Users should not instantiate this class."""
+    Users should not instantiate this class.
+
+    Attributes:
+        event_type_value (str): Value for a DagsterEventType.
+        pipeline_name (str)
+        step_key (str)
+        solid_handle (SolidHandle)
+        step_kind_value (str): Value for a StepKind.
+        logging_tags (Dict[str, str])
+        event_specific_data (Any): Type must correspond to event_type_value.
+        message (str)
+        pid (int)
+    """
 
     @staticmethod
     def from_step(event_type, step_context, event_specific_data=None, message=None):
@@ -397,6 +409,11 @@ class DagsterEvent(
         )
         return self.event_specific_data
 
+    @property
+    def pipeline_failure_data(self):
+        _assert_type("pipeline_failure_data", DagsterEventType.PIPELINE_FAILURE, self.event_type)
+        return self.event_specific_data
+
     @property
     def engine_event_data(self):
         _assert_type("engine_event_data", DagsterEventType.ENGINE_EVENT, self.event_type)
@@ -428,7 +445,7 @@ class DagsterEvent(
                 output_name=step_output_data.step_output_handle.output_name,
                 output_type=step_context.step.step_output_named(
                     step_output_data.step_output_handle.output_name
-                ).dagster_type.name,
+                ).dagster_type.display_name,
                 type_check_clause=(
                     " Warning! Type check failed."
                     if not step_output_data.type_check_data.success
@@ -472,7 +489,7 @@ class DagsterEvent(
                 input_name=step_input_data.input_name,
                 input_type=step_context.step.step_input_named(
                     step_input_data.input_name
-                ).dagster_type.name,
+                ).dagster_type.display_name,
                 type_check_clause=(
                     " Warning! Type check failed."
                     if not step_input_data.type_check_data.success
@@ -586,12 +603,17 @@ class DagsterEvent(
         )
 
     @staticmethod
-    def pipeline_failure(pipeline_context):
+    def pipeline_failure(pipeline_context, context_msg, error_info=None):
+
         return DagsterEvent.from_pipeline(
             DagsterEventType.PIPELINE_FAILURE,
             pipeline_context,
-            message='Execution of pipeline "{pipeline_name}" failed.'.format(
-                pipeline_name=pipeline_context.pipeline_def.name
+            message='Execution of pipeline "{pipeline_name}" failed. {context_msg}'.format(
+                pipeline_name=pipeline_context.pipeline_def.name,
+                context_msg=check.str_param(context_msg, "context_msg"),
+            ),
+            event_specific_data=PipelineFailureData(
+                check.opt_inst_param(error_info, "error_info", SerializableErrorInfo)
             ),
         )
 
@@ -936,6 +958,14 @@ class PipelineInitFailureData(namedtuple("_PipelineInitFailureData", "error")):
         )
 
 
+@whitelist_for_serdes
+class PipelineFailureData(namedtuple("_PipelineFailureData", "error")):
+    def __new__(cls, error):
+        return super(PipelineFailureData, cls).__new__(
+            cls, error=check.opt_inst_param(error, "error", SerializableErrorInfo)
+        )
+
+
 @whitelist_for_serdes
 class HookErroredData(namedtuple("_HookErroredData", "error")):
     def __new__(cls, error):
