@@ -203,7 +203,7 @@ class ExecutionStep(
         "_ExecutionStep",
         (
             "pipeline_name key_suffix step_inputs step_input_dict step_outputs step_output_dict "
-            "compute_fn kind solid_handle solid_version logging_tags tags hook_defs"
+            "compute_fn kind solid_handle solid_version solid_required_resource_keys logging_tags tags hook_defs"
         ),
     )
 ):
@@ -236,6 +236,7 @@ class ExecutionStep(
             kind=check.inst_param(kind, "kind", StepKind),
             solid_handle=check.inst_param(solid_handle, "solid_handle", SolidHandle),
             solid_version=solid.definition.version,
+            solid_required_resource_keys=solid.definition.required_resource_keys,
             logging_tags=merge_dicts(
                 {
                     "step_key": str(solid_handle) + "." + key_suffix,
