@@ -28,7 +28,7 @@ import six
 import yaml
 
 from dagster import check
-from dagster.core.definitions.executable import ExecutablePipeline
+from dagster.core.definitions.pipeline_base import IPipeline
 from dagster.core.definitions.reconstructable import (
     ReconstructablePipeline,
     ReconstructableRepository,
@@ -49,11 +49,13 @@ TELEMETRY_VERSION = "0.2"
 
 # When adding to TELEMETRY_WHITELISTED_FUNCTIONS, please also update the literalinclude in
 # docs/next/src/pages/install/index.mdx
+# start_TELEMETRY_WHITELISTED_FUNCTIONS
 TELEMETRY_WHITELISTED_FUNCTIONS = {
     "execute_execute_command",
     "_logged_execute_pipeline",
     "execute_launch_command",
 }
+# end_TELEMETRY_WHITELISTED_FUNCTIONS
 
 
 def telemetry_wrapper(f):
@@ -360,7 +362,7 @@ def log_external_repo_stats(instance, source, external_repo, external_pipeline=N
 def log_repo_stats(instance, source, pipeline=None, repo=None):
     check.inst_param(instance, "instance", DagsterInstance)
     check.str_param(source, "source")
-    check.opt_inst_param(pipeline, "pipeline", ExecutablePipeline)
+    check.opt_inst_param(pipeline, "pipeline", IPipeline)
     check.opt_inst_param(repo, "repo", ReconstructableRepository)
 
     if _get_instance_telemetry_enabled(instance):
