@@ -23,8 +23,8 @@ from dagster import check, seven
 from dagster.core.errors import DagsterInvariantViolationError
 from dagster.seven import IS_WINDOWS, TemporaryDirectory, multiprocessing, thread
 from dagster.seven.abc import Mapping
-from dagster.utils.merger import merge_dicts
 
+from .merger import merge_dicts
 from .yaml_utils import load_yaml_from_glob_list, load_yaml_from_globs, load_yaml_from_path
 
 if sys.version_info > (3,):
@@ -189,6 +189,9 @@ class frozenlist(list):
     reverse = __readonly__
     sort = __readonly__
 
+    def __hash__(self):
+        return hash(tuple(self))
+
 
 def make_readonly_value(value):
     if isinstance(value, list):
